@@ -5,7 +5,7 @@
 
     Command line interface.
 
-    :copyright: 2006-2007 by Georg Brandl.
+    :copyright: 2006-2008 by Georg Brandl.
     :license: BSD, see LICENSE for more details.
 """
 import sys
@@ -15,7 +15,7 @@ from textwrap import dedent
 from pygments import __version__, __author__, highlight
 from pygments.util import ClassNotFound, OptionError, docstring_headline
 from pygments.lexers import get_all_lexers, get_lexer_by_name, get_lexer_for_filename, \
-     find_lexer_class
+     find_lexer_class, guess_lexer, TextLexer
 from pygments.formatters import get_all_formatters, get_formatter_by_name, \
      get_formatter_for_filename, find_formatter_class, \
      TerminalFormatter  # pylint:disable-msg=E0611
@@ -24,10 +24,10 @@ from pygments.styles import get_all_styles, get_style_by_name
 
 
 USAGE = """\
-Usage: %s [-l <lexer>] [-F <filter>[:<options>]] [-f <formatter>]
-          [-O <options>] [-o <outfile>] [<infile>]
+Usage: %s [-l <lexer> | -g] [-F <filter>[:<options>]] [-f <formatter>]
+          [-O <options>] [-P <option=value>] [-o <outfile>] [<infile>]
 
-       %s -S <style> -f <formatter> [-a <arg>] [-O <options>]
+       %s -S <style> -f <formatter> [-a <arg>] [-O <options>] [-P <option=value>]
        %s -L [<which> ...]
        %s -H <type> <name>
        %s -h | -V
@@ -38,7 +38,9 @@ If no input file is given, use stdin, if -o is not given, use stdout.
 
 <lexer> is a lexer name (query all lexer names with -L). If -l is not
 given, the lexer is guessed from the extension of the input file name
-(this obviously doesn't work if the input is stdin).
+(this obviously doesn't work if the input is stdin).  If -g is passed,
+attempt to guess the lexer from the file contents, or pass through as
+plain text if this fails (this can work for stdin).
 
 Likewise, <formatter> is a formatter name, and will be guessed from
 the extension of the output file name. If no output file is given,
@@ -47,11 +49,16 @@ the terminal formatter will be used by default.
 With the -O option, you can give the lexer and formatter a comma-
 separated list of options, e.g. ``-O bg=light,python=cool``.
 
+The -P option adds lexer and formatter options like the -O option, but
+you can only give one option per -P. That way, the option value may
+contain commas and equals signs, which it can't with -O, e.g.
+``-P "heading=Pygments, the Python highlighter".
+
 With the -F option, you can add filters to the token stream, you can
 give options in the same way as for -O after a colon (note: there must
 not be spaces around the colon).
 
-The -O and -F options can be given multiple times.
+The -O, -P and -F options can be given multiple times.
 
 With the -S option, print out style definitions for style <style>
 for formatter <formatter>. The argument given by -a is formatter
@@ -117,7 +124,7 @@ def _print_help(what, name):
             cls = find_filter_class(name)
             print "Help on the %s filter:" % name
             print dedent(cls.__doc__)
-    except ClassNotFound:
+    except AttributeError:
         print >>sys.stderr, "%s not found!" % what
 
 
@@ -172,7 +179,7 @@ def _print_list(what):
             print "    %s" % docstring_headline(cls)
 
 
-def main(args):
+def main(args=sys.argv):
     """
     Main command line entry point.
     """
@@ -181,16 +188,19 @@ def main(args):
     usage = USAGE % ((args[0],) * 5)
 
     try:
-        popts, args = getopt.getopt(args[1:], "l:f:F:o:O:LS:a:hVH")
+        popts, args = getopt.getopt(args[1:], "l:f:F:o:O:P:LS:a:hVHg")
     except getopt.GetoptError, err:
         print >>sys.stderr, usage
         return 2
     opts = {}
     O_opts = []
+    P_opts = []
     F_opts = []
     for opt, arg in popts:
         if opt == '-O':
             O_opts.append(arg)
+        elif opt == '-P':
+            P_opts.append(arg)
         elif opt == '-F':
             F_opts.append(arg)
         opts[opt] = arg
@@ -204,7 +214,7 @@ def main(args):
         return 0
 
     if opts.pop('-V', None) is not None:
-        print 'Pygments version %s, (c) 2006 by %s.' % (__version__, __author__)
+        print 'Pygments version %s, (c) 2006-2008 by %s.' % (__version__, __author__)
         return 0
 
     # handle ``pygmentize -L``
@@ -238,9 +248,18 @@ def main(args):
         return 0
 
     # parse -O options
-    O_opts = _parse_options(O_opts)
-    # parse -F options
-    F_opts = _parse_filters(F_opts)
+    parsed_opts = _parse_options(O_opts)
+    opts.pop('-O', None)
+
+    # parse -P options
+    for p_opt in P_opts:
+        try:
+            name, value = p_opt.split('=', 1)
+        except ValueError:
+            parsed_opts[p_opt] = True
+        else:
+            parsed_opts[name] = value
+    opts.pop('-P', None)
 
     # handle ``pygmentize -S``
     S_opt = opts.pop('-S', None)
@@ -255,8 +274,8 @@ def main(args):
             return 2
 
         try:
-            O_opts['style'] = S_opt
-            fmter = get_formatter_by_name(f_opt, **O_opts)
+            parsed_opts['style'] = S_opt
+            fmter = get_formatter_by_name(f_opt, **parsed_opts)
         except ClassNotFound, err:
             print >>sys.stderr, err
             return 1
@@ -270,12 +289,16 @@ def main(args):
         print >>sys.stderr, usage
         return 2
 
+    # parse -F options
+    F_opts = _parse_filters(F_opts)
+    opts.pop('-F', None)
+
     # select formatter
     outfn = opts.pop('-o', None)
     fmter = opts.pop('-f', None)
     if fmter:
         try:
-            fmter = get_formatter_by_name(fmter, **O_opts)
+            fmter = get_formatter_by_name(fmter, **parsed_opts)
         except (OptionError, ClassNotFound), err:
             print >>sys.stderr, 'Error:', err
             return 1
@@ -283,7 +306,7 @@ def main(args):
     if outfn:
         if not fmter:
             try:
-                fmter = get_formatter_for_filename(outfn, **O_opts)
+                fmter = get_formatter_for_filename(outfn, **parsed_opts)
             except (OptionError, ClassNotFound), err:
                 print >>sys.stderr, 'Error:', err
                 return 1
@@ -294,14 +317,14 @@ def main(args):
             return 1
     else:
         if not fmter:
-            fmter = TerminalFormatter(**O_opts)
+            fmter = TerminalFormatter(**parsed_opts)
         outfile = sys.stdout
 
     # select lexer
     lexer = opts.pop('-l', None)
     if lexer:
         try:
-            lexer = get_lexer_by_name(lexer, **O_opts)
+            lexer = get_lexer_by_name(lexer, **parsed_opts)
         except (OptionError, ClassNotFound), err:
             print >>sys.stderr, 'Error:', err
             return 1
@@ -312,23 +335,53 @@ def main(args):
             return 2
 
         infn = args[0]
-        if not lexer:
-            try:
-                lexer = get_lexer_for_filename(infn, **O_opts)
-            except (OptionError, ClassNotFound), err:
-                print >>sys.stderr, 'Error:', err
-                return 1
-
         try:
             code = file(infn).read()
         except Exception, err:
             print >>sys.stderr, 'Error: cannot read infile:', err
             return 1
-    else:
+
         if not lexer:
-            print >>sys.stderr, 'Error: no lexer name given and reading from stdin'
+            try:
+                lexer = get_lexer_for_filename(infn, **parsed_opts)
+            except ClassNotFound, err:
+                if '-g' in opts:
+                    try:
+                        lexer = guess_lexer(code)
+                    except ClassNotFound:
+                        lexer = TextLexer()
+                else:
+                    print >>sys.stderr, 'Error:', err
+                    return 1
+            except OptionError, err:
+                print >>sys.stderr, 'Error:', err
+                return 1
+
+    else:
+        if '-g' in opts:
+            code = sys.stdin.read()
+            try:
+                lexer = guess_lexer(code)
+            except ClassNotFound:
+                lexer = TextLexer()
+        elif not lexer:
+            print >>sys.stderr, 'Error: no lexer name given and reading ' + \
+                                'from stdin (try using -g or -l <lexer>)'
             return 2
-        code = sys.stdin.read()
+        else:
+            code = sys.stdin.read()
+
+    # No encoding given? Use latin1 if output file given,
+    # stdin/stdout encoding otherwise.
+    # (This is a compromise, I'm not too happy with it...)
+    if 'encoding' not in parsed_opts and 'outencoding' not in parsed_opts:
+        if outfn:
+            # encoding pass-through
+            fmter.encoding = 'latin1'
+        else:
+            # use terminal encoding
+            lexer.encoding = getattr(sys.stdin, 'encoding', None) or 'ascii'
+            fmter.encoding = getattr(sys.stdout, 'encoding', None) or 'ascii'
 
     # ... and do it!
     try:
