@@ -5,15 +5,20 @@
 
     Formatter for HTML output.
 
-    :copyright: 2006-2007 by Georg Brandl, Armin Ronacher.
+    :copyright: 2006-2008 by Georg Brandl, Armin Ronacher.
     :license: BSD, see LICENSE for more details.
 """
 import sys, os
 import StringIO
 
+try:
+    set
+except NameError:
+    from sets import Set as set
+
 from pygments.formatter import Formatter
 from pygments.token import Token, Text, STANDARD_TYPES
-from pygments.util import get_bool_opt, get_int_opt
+from pygments.util import get_bool_opt, get_int_opt, get_list_opt
 
 
 __all__ = ['HtmlFormatter']
@@ -129,6 +134,9 @@ class HtmlFormatter(Formatter):
 
     Wrapping can be disabled using the `nowrap` option.
 
+    A list of lines can be specified using the `hl_lines` option to make these
+    lines highlighted (as of Pygments 0.11).
+
     With the `full` option, a complete HTML 4 document is output, including
     the style definitions inside a ``<style>`` tag, or in a separate file if
     the `cssfile` option is given.
@@ -204,6 +212,10 @@ class HtmlFormatter(Formatter):
     `cssstyles`
         Inline CSS styles for the wrapping ``<div>`` tag (default: ``''``).
 
+    `prestyles`
+        Inline CSS styles for the ``<pre>`` tag (default: ``''``).  *New in
+        Pygments 0.11.*
+
     `cssfile`
         If the `full` option is true and this option is given, it must be the
         name of an external file. If the filename does not include an absolute
@@ -226,11 +238,14 @@ class HtmlFormatter(Formatter):
         The default value is ``False``, which means no line numbers at all.
 
         **Note:** with the default ("table") line number mechanism, the line
-        numbers and code will have different line heights in Internet Explorer
+        numbers and code can have different line heights in Internet Explorer
         unless you give the enclosing ``<pre>`` tags an explicit ``line-height``
         CSS property (you get the default line spacing with ``line-height:
         125%``).
 
+    `hl_lines`
+        Specify a list of lines to be highlighted.  *New in Pygments 0.11.*
+
     `linenostart`
         The line number for the first line (default: ``1``).
 
@@ -320,6 +335,7 @@ class HtmlFormatter(Formatter):
         self.classprefix = options.get('classprefix', '')
         self.cssclass = options.get('cssclass', 'highlight')
         self.cssstyles = options.get('cssstyles', '')
+        self.prestyles = options.get('prestyles', '')
         self.cssfile = options.get('cssfile', '')
         linenos = options.get('linenos', False)
         if linenos == 'inline':
@@ -335,6 +351,12 @@ class HtmlFormatter(Formatter):
         self.nobackground = get_bool_opt(options, 'nobackground', False)
         self.lineseparator = options.get('lineseparator', '\n')
         self.lineanchors = options.get('lineanchors', '')
+        self.hl_lines = set()
+        for lineno in get_list_opt(options, 'hl_lines', []):
+            try:
+                self.hl_lines.add(int(lineno))
+            except ValueError:
+                pass
 
         self._class_cache = {}
         self._create_stylesheet()
@@ -405,6 +427,9 @@ class HtmlFormatter(Formatter):
                 text_style = ' ' + self.class2style[self.ttype2class[Text]][0]
             lines.insert(0, '%s { background: %s;%s }' %
                          (prefix(''), self.style.background_color, text_style))
+        if self.style.highlight_color is not None:
+            lines.insert(0, '%s.hll { background-color: %s }' %
+                         (prefix(''), self.style.highlight_color))
         return '\n'.join(lines)
 
     def _wrap_full(self, inner, outfile):
@@ -512,7 +537,8 @@ class HtmlFormatter(Formatter):
         yield 0, '</div>\n'
 
     def _wrap_pre(self, inner):
-        yield 0, '<pre>'
+        yield 0, ('<pre'
+                  + (self.prestyles and ' style="%s"' % self.prestyles) + '>')
         for tup in inner:
             yield tup
         yield 0, '</pre>'
@@ -576,6 +602,21 @@ class HtmlFormatter(Formatter):
         if line:
             yield 1, line + (lspan and '</span>') + lsep
 
+    def _highlight_lines(self, tokensource):
+        """
+        Highlighted the lines specified in the `hl_lines` option by
+        post-processing the token stream coming from `_format_lines`.
+        """
+        hls = self.hl_lines
+
+        for i, (t, value) in enumerate(tokensource):
+            if t != 1:
+                yield t, value
+            if i + 1 in hls: # i + 1 because Python indexes start at 0
+                yield 1, '<span class="hll">%s</span>' % value
+            else:
+                yield 1, value
+
     def wrap(self, source, outfile):
         """
         Wrap the ``source``, which is a generator yielding
@@ -599,6 +640,8 @@ class HtmlFormatter(Formatter):
         linewise, e.g. line number generators.
         """
         source = self._format_lines(tokensource)
+        if self.hl_lines:
+            source = self._highlight_lines(source)
         if not self.nowrap:
             if self.linenos == 2:
                 source = self._wrap_inlinelinenos(source)
