@@ -15,6 +15,7 @@ import random
 from pygments import lexers, formatters, filters, format
 from pygments.token import _TokenType, Text
 from pygments.lexer import RegexLexer
+from pygments.formatters.img import FontNotFound
 
 test_content = [chr(i) for i in xrange(33, 128)] * 5
 random.shuffle(test_content)
@@ -40,6 +41,7 @@ class LexersTest(unittest.TestCase):
             result = lexer.analyse_text("abc")
             a(isinstance(result, float) and 0.0 <= result <= 1.0)
 
+            inst = lexer(opt1="val1", opt2="val2")
             if issubclass(lexer, RegexLexer):
                 if not hasattr(lexer, '_tokens'):
                     # if there's no "_tokens", the lexer has to be one with
@@ -50,7 +52,6 @@ class LexersTest(unittest.TestCase):
                 else:
                     a('root' in lexer._tokens, '%s has no root state' % lexer)
 
-            inst = lexer(opt1="val1", opt2="val2")
             tokens = list(inst.get_tokens(test_content))
             txt = ""
             for token in tokens:
@@ -131,7 +132,10 @@ class FormattersTest(unittest.TestCase):
             a(info[1], "missing formatter aliases") # aliases
             a(info[3], "missing formatter docstring") # doc
 
-            inst = formatter(opt1="val1")
+            try:
+                inst = formatter(opt1="val1")
+            except (ImportError, FontNotFound):
+                continue
             inst.get_style_defs()
             inst.format(ts, out)
 
@@ -163,7 +167,11 @@ class FormattersTest(unittest.TestCase):
         # test that the formatter supports encoding and Unicode
         tokens = list(lexers.PythonLexer(encoding='utf-8').get_tokens("def f(): 'ä'"))
         for formatter, info in formatters.FORMATTERS.iteritems():
-            inst = formatter(encoding=None)
+            try:
+                inst = formatter(encoding=None)
+            except (ImportError, FontNotFound):
+                # some dependency or font not installed
+                continue
             out = format(tokens, inst)
             if formatter.unicodeoutput:
                 self.assert_(type(out) is unicode)
